// Copyright (c) 2019-present,  SurfStudio LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'dart:convert';
import 'dart:io';

import 'package:path/path.dart' as p;
import 'package:yaml/yaml.dart';

/// Test coverage tool.
/// Join dart files links for reimport, run tests and collect coverage,
/// transform lcov file into human readable html.
/// Have to been run in root directory of flutter project.
Future<void> main() async {
  final pubspecFile = File('pubspec.yaml');
  if (!pubspecFile.existsSync()) {
    print('Pubspec file not exist');
    exit(1);
  }
  final dataFromPubspec = pubspecFile.readAsStringSync();

  final String projectName;
  try {
    final parsedData = loadYaml(dataFromPubspec);
    projectName = parsedData['name'];
  } catch (YamlException) {
    print('Pubspec parse was failed');
    exit(1);
  }

  final directoryLib = Directory('lib');
  if (!directoryLib.existsSync()) {
    print('Directory lib not found');
    exit(1);
  }

  final listFiles = Directory('lib').listSync(recursive: true);
  var preparedList = <FileSystemEntity?>[];
  preparedList = listFiles.where(
    (element) {
      return element is File &&
          element.path.contains('.dart') &&
          !element.path.contains('.g.') &&
          !_fileDoesNotFit(element);
    },
  ).toList();

  await Directory('test').create();

  final preparedString = StringBuffer();
  for (final file in preparedList) {
    if (file != null) {
      preparedString.write(
          "export 'package:$projectName/${p.relative(file.path.replaceAll('lib/', ''))}';\n");
    }
  }
  preparedString.write('void main() {}');

  File('test/all_files_test.dart').createSync();
  await File('test/all_files_test.dart')
      .writeAsBytes(utf8.encode(preparedString.toString()));

  await Process.run('flutter', ['test', '--coverage']);

  try {
    await Process.run(
      'genhtml',
      [
        'coverage/lcov.info',
        '-o',
        'coverage/html',
      ],
    );
  } catch (ProcessException) {
    print(ProcessException.toString());
  }

  File('test/all_files_test.dart').delete();
}

bool _fileDoesNotFit(File file) {
  final fileToString = file.readAsStringSync();
  return fileToString.contains("part of '") ||
      fileToString.contains("part of'") ||
      fileToString.contains("part '") ||
      fileToString.contains("part'") ||
      fileToString.contains("export '") ||
      fileToString.contains("export'") ||
      fileToString.contains('main()');
}
